@@ -86,8 +86,10 @@ class StoreSnapshot(Generic[State]):
             )
             raise RuntimeError(msg)
 
+        from pathlib import Path
+
         filename = self.get_filename(title)
-        path = self.results_dir / filename
+        path = Path(self.results_dir / filename)
         json_path = path.with_suffix('.jsonc')
         mismatch_path = path.with_suffix('.mismatch.jsonc')
 
